SystemOrganization addCategory: #'Tweak-Morphic'!
SystemOrganization addCategory: #'Tweak-Morphic-DevTools'!

!MenuMorph methodsFor: '*Tweak-Morphic' stamp: 'ar 5/5/2004 18:18'!
openAt: aPoint
	self position: aPoint.
	self invokeModal.! !

Project subclass: #CProjectInMorphic
	instanceVariableNames: 'uiProcess tweakWorld'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Morphic'!

!CProjectInMorphic class methodsFor: 'class initialization' stamp: 'ar 6/18/2005 16:42'!
initialize
	"CProjectInMorphic initialize"
	TheWorldMenu unregisterOpenCommandWithReceiver: self.! !

!CProjectInMorphic class methodsFor: 'accessing' stamp: 'ar 12/18/2004 20:04'!
new
	| new |
	new := self basicNew.
	self addingProject: new.
	new initTweak.
	^new! !

!CProjectInMorphic class methodsFor: 'accessing' stamp: 'ar 6/18/2005 15:05'!
open
	(ProjectViewMorph on: CTweakProjectInMorphic new) openInWorld! !

!CProjectInMorphic methodsFor: 'entering' stamp: 'ar 12/18/2004 21:37'!
enter: returningFlag revert: revertFlag saveForRevert: saveForRevert
	"Install my ChangeSet, Transcript, and scheduled views as current globals. If returningFlag is true, we will return to the project from whence the current project was entered; don't change its previousProject link in this case."
	(revertFlag or:[saveForRevert]) ifTrue:[^self error: 'Won''t do this'].
	self isCurrentProject ifTrue: [^ self].
	"Check the guards"
	guards ifNotNil:
		[guards := guards reject: [:obj | obj isNil].
		guards do: [:obj | obj okayToEnterProject ifFalse: [^ self]]].
	CurrentProject finalExitActions.
	ProjectHistory remember: CurrentProject.

	"Update the display depth and make a thumbnail of the current project"
	CurrentProject displayDepth: Display depth.

	"Show the project transition.
	Note: The project zoom is run in the context of the old project,
		so that eventual errors can be handled accordingly"
	displayDepth == nil ifTrue: [displayDepth := Display depth].
	self installNewDisplay: Display extent depth: displayDepth.
	(self showZoom) ifTrue: [
		self displayZoom: CurrentProject parent ~~ self].
	Smalltalk at: #ScorePlayer ifPresentAndInMemory:[:playerClass | 
		playerClass allSubInstancesDo:[:player | player pause]].
	returningFlag
			ifTrue: [nextProject := CurrentProject]
			ifFalse: [previousProject := CurrentProject].
	CurrentProject saveState.
	CurrentProject := self.
	self installProjectPreferences.
	ChangeSet  newChanges: changeSet.
	TranscriptStream newTranscript: transcript.
	Sensor flushKeyboard.
	Smalltalk isMorphic ifTrue: [World pauseEventRecorder].
	ProjectHistory remember: CurrentProject.
	self finalEnterActions.
	uiProcess := [self run] forkAt: Processor userSchedulingPriority.
	Processor terminateActive.! !

!CProjectInMorphic methodsFor: 'entering' stamp: 'ar 12/27/2004 11:00'!
finalEnterActions
	"Perform the final actions necessary as the receiver project is entered"
	World := nil.
	Sensor installInterruptWatcher.
	Smalltalk installLowSpaceWatcher.
	UIManager default: TweakUIManager new.! !

!CProjectInMorphic methodsFor: 'entering' stamp: 'ar 1/5/2005 17:51'!
finalExitActions
	"From script process this needs to be forked off, e.g.,
		sema := Semaphore new.
		[Project current previousProject
		sema signal] fork.
		sema wait.
	or so..."
	Processor activeProcess isScriptingProcess 
		ifTrue:[^self error:'Please don''t do this'].
	uiProcess ifNotNil:[uiProcess terminate].
	tweakWorld display: (Form extent: 32@32 depth: 32).
	UIManager default: nil.
	super finalExitActions.
! !

!CProjectInMorphic methodsFor: 'entering' stamp: 'ar 12/18/2004 21:45'!
imageFormOfSize: extentPoint depth: d
	| oldDisplay newDisplay |
	oldDisplay := tweakWorld display.
	newDisplay := DisplayScreen extent: extentPoint depth: d.
	tweakWorld display: newDisplay.
	tweakWorld displayWorld.
	tweakWorld display: oldDisplay.
	^newDisplay! !

!CProjectInMorphic methodsFor: 'initialize' stamp: 'ar 6/21/2010 11:23'!
initTweak
	| script |
	changeSet := ChangeSet new.
	transcript := Smalltalk at: #CTranscriptStream ifPresent:[:cls| cls new].
	displayDepth := Display depth.
	parentProject := CurrentProject.
	tweakWorld := CWorldPlayer new.
	script := tweakWorld startScript:[tweakWorld onBootstrap].
	script setScheduler: tweakWorld scheduler.
	script resume.! !

!CProjectInMorphic methodsFor: 'entering' stamp: 'ar 12/21/2004 00:58'!
leaveTo: aProject
	"Leave this project entering aProject"
	| sema |
	Processor activeProcess isScriptingProcess ifTrue:[
		"Cannot do this from the inside"
		[self leaveTo: aProject] fork.
		^self].
	sema := Semaphore new.
	tweakWorld doneSema: sema.
	sema wait.
	aProject enter.
! !

!CProjectInMorphic methodsFor: 'entering' stamp: 'ar 12/18/2004 20:14'!
makeThumbnail
	"Make a thumbnail image of this project from the Display."
	tweakWorld displayWorld. "clean pending damage"
	viewSize ifNil: [viewSize := Display extent // 8].
	thumbnail := Form extent: viewSize depth: Display depth.
	(WarpBlt current toForm: thumbnail)
			sourceForm: Display;
			cellSize: 2;  "installs a colormap"
			combinationRule: Form over;
			copyQuad: (Display boundingBox) innerCorners
			toRect: (0@0 extent: viewSize).
	InternalThreadNavigationMorph cacheThumbnailFor: self.
	^thumbnail
! !

!CProjectInMorphic methodsFor: 'entering' stamp: 'ar 12/21/2004 01:06'!
restart
	"restart the hard way"
	uiProcess ifNotNil:[uiProcess terminate].
	uiProcess := [self run] forkAt: Processor userSchedulingPriority.! !

!CProjectInMorphic methodsFor: 'entering' stamp: 'ar 12/18/2004 21:30'!
run
	tweakWorld run.! !

!CProjectInMorphic methodsFor: 'entering' stamp: 'ar 12/18/2004 21:31'!
saveState
	"Save the current state in me prior to leaving this project"
	changeSet := ChangeSet current.
	thumbnail ifNotNil: [thumbnail hibernate].
	Sensor flushAllButDandDEvents. "Will be reinstalled by World>>install"
	transcript := Transcript.
	uiProcess := nil.! !

!Project methodsFor: '*Tweak-Morphic' stamp: 'ar 4/22/2005 11:52'!
transcript
	^transcript! !

LazyListMorph subclass: #TweakMessageListMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Morphic-DevTools'!

!TweakMessageListMorph methodsFor: 'drawing' stamp: 'ar 5/9/2005 17:03'!
display: item  atRow: row on: canvas
	"display the given item at row row"
	| drawAction box args |
	drawAction := listSource drawAction ifNil:[^super display: item  atRow: row on: canvas].
	box := (self drawBoundsForRow: row) intersect: self bounds.
	args := Array new: 3.
	args at: 1 put: row.
	args at: 2 put: box.
	args at: 3 put: canvas.
	drawAction valueWithArguments: args.
! !

!TweakMessageListMorph methodsFor: 'drawing' stamp: 'ar 5/9/2005 17:02'!
drawOn: aCanvas
	| drawAction args box |
	listItems size = 0 ifTrue: [ ^self ].
	drawAction := listSource drawAction ifNil:[^super drawOn: aCanvas].
	args := Array new: 3.
	args at: 3 put: aCanvas.
 	self drawSelectionOn: aCanvas.
	(self topVisibleRowForCanvas: aCanvas) to: (self bottomVisibleRowForCanvas: aCanvas) do: [ :row |
		(listSource itemSelectedAmongMultiple:  row) ifTrue: [
			self drawBackgroundForMulti: row on: aCanvas. ].
		box := (self drawBoundsForRow: row) intersect: self bounds.
		args at: 1 put: row.
		args at: 2 put: box.
		drawAction valueWithArguments: args.
	].

	listSource potentialDropRow > 0 ifTrue: [
		self highlightPotentialDropRow: listSource potentialDropRow on: aCanvas ].! !

!MVCMenuMorph methodsFor: '*tweak-morphic-override' stamp: 'bf 11/26/2004 10:53'!
displayAt: aPoint during: aBlock
	"Add this menu to the Morphic world during the execution of the given block."
	
	self ifTweak: [^CMenu displayMVCMenu: self at: aPoint during: aBlock].

	Smalltalk isMorphic ifFalse: [^ self].

	ActiveWorld addMorph: self centeredNear: aPoint.
	self world displayWorld.  "show myself"
	aBlock value.
	self delete! !

!MVCMenuMorph methodsFor: '*tweak-morphic-override' stamp: 'ar 1/22/2005 15:47'!
informUserAt: aPoint during: aBlock
	"Add this menu to the Morphic world during the execution of the given block."

	| title w |
	Smalltalk isMorphic ifFalse: [^ self].

	title := self allMorphs detect: [ :ea | ea hasProperty: #titleString ].
	title := title submorphs first.
	self visible: false.
	w := ActiveWorld.
	^[aBlock value:[:string|
		self visible ifFalse:[
			w addMorph: self centeredNear: aPoint.
			self visible: true].
		title contents: string.
		self setConstrainedPosition: Sensor cursorPoint hangOut: false.
		self changed.
		w displayWorld		 "show myself"
	]] ensure:[
		self delete.
		w displayWorld
	].! !

!MVCMenuMorph methodsFor: '*tweak-morphic-override' stamp: 'bf 11/29/2004 15:36'!
invokeAt: aPoint in: aWorld allowKeyboard: aBoolean
	"Add this menu to the given world centered at the given point. Wait for the user to make a selection and answer it. The selection value returned is an integer in keeping with PopUpMenu, if the menu is converted from an MVC-style menu."
	"Details: This is invoked synchronously from the caller. In order to keep processing inputs and updating the screen while waiting for the user to respond, this method has its own version of the World's event loop." 
	| w |
	self flag: #bob.		"is <aPoint> global or local?"
	self flag: #arNote.	"<aPoint> is local to aWorld"
	self ifTweak: [
		CMenu invokeMVCMenu: self at: aPoint.
	] ifNot: [
		self popUpAt: aPoint forHand: aWorld primaryHand in: aWorld allowKeyboard: aBoolean.
		done := false.
		w := aWorld outermostWorldMorph. "containing hand"
		[self isInWorld & done not] whileTrue: [w doOneSubCycle].
		self delete.
	].
	^ mvcSelection
! !

!ControlManager class methodsFor: '*Tweak-Morphic-override' stamp: 'ar 12/21/2004 11:54'!
shutDown  "Saves space in snapshots"
	self ifTweak:[^self].
	Smalltalk isMorphic ifFalse: [ScheduledControllers unCacheWindows]! !

!ControlManager class methodsFor: '*Tweak-Morphic-override' stamp: 'ar 12/21/2004 12:01'!
startUp
	self ifTweak:[^self].
	Smalltalk isMorphic ifFalse: [ScheduledControllers restore]! !

!Workspace methodsFor: '*Tweak-Morphic-override' stamp: 'ar 6/18/2010 18:34'!
addModelItemsToWindowMenu: aMenu 
	
	aMenu addLine.
	aMenu
		add: 'save as HTML...' target: self action: #saveHtmlInFile;
		add: 'save as Blog HTML...' target: self action: #saveBlogHtmlInFile;
		add: 'save contents to file...' target: self action: #saveContentsInFile;
		add: 'reset variables' target: self action: #initializeBindings.
	aMenu addLine.
	aMenu
		addUpdating: #mustDeclareVariableWording
		target: self
		action: #toggleVariableDeclarationMode.
	aMenu
		addUpdating: #acceptDroppedMorphsWording
		target: self
		action: #toggleDroppingMorphForReference.

	self addToggleStylingMenuItemTo: aMenu.
! !

!Workspace methodsFor: '*Tweak-Morphic-override' stamp: 'eem 6/14/2008 13:50'!
saveBlogHtmlInFile
	"A bit of a hack to pass along this message to the controller or morph.  (Possibly this Workspace menu item could be deleted, since it's now in the text menu.)"
	(self dependents detect: [:dep | dep isKindOf: PluggableTextMorph] ifNone: [nil]) ifNotNil:
		[:textMorph| textMorph saveHtmlInFileUseBreaks: false]! !

!Workspace methodsFor: '*Tweak-Morphic-override' stamp: 'eem 6/14/2008 13:53'!
saveHtmlInFile
	"A bit of a hack to pass along this message to the controller or morph.  (Possibly this Workspace menu item could be deleted, since it's now in the text menu.)"

	(self dependents detect: [:dep | dep isKindOf: PluggableTextMorph] ifNone: [nil]) ifNotNil:
		[:textMorph| textMorph saveHtmlInFileUseBreaks: true]! !

!Browser methodsFor: '*Tweak-Morphic-override' stamp: 'ar 7/19/2010 11:38'!
classDefinitionText
	"return the text to display for the definition of the currently selected class"
	^self selectedClassOrMetaClass 
		ifNil: [''] 
		ifNotNil: [:aClass| aClass browserDefinition: #default]! !

Browser subclass: #TweakBrowser
	instanceVariableNames: 'messageListCache overriddenClassCache'
	classVariableNames: 'BothArrows ChangeEventIcon DefaultEventIcon DownArrow EventMap FlagIcon KeyboardEventIcon MouseEventIcon NoEventIcon StructureEventIcon Temp2EventIcon TempEventIcon TextEventIcon TickEventIcon UpArrow ViewEventIcon WarningIcon WindowEventIcon'
	poolDictionaries: ''
	category: 'Tweak-Morphic-DevTools'!

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 2/12/2010 19:39'!
bothArrowIcon
	^BothArrows ifNil:[
		BothArrows := CIconLibrary formFromEncodedImage: 
'R0lGODlhCwALAPcAAAAAAAAAMwAAZgAAmQAAzAAA/zMAADMAMzMAZjMAmTMAzDMA/2YAAGYA
M2YAZmYAmWYAzGYA/5kAAJkAM5kAZpkAmZkAzJkA/8wAAMwAM8wAZswAmcwAzMwA//8AAP8A
M/8AZv8Amf8AzP8A/wAzAAAzMwAzZgAzmQAzzAAz/zMzADMzMzMzZjMzmTMzzDMz/2YzAGYz
M2YzZmYzmWYzzGYz/5kzAJkzM5kzZpkzmZkzzJkz/8wzAMwzM8wzZswzmcwzzMwz//8zAP8z
M/8zZv8zmf8zzP8z/wBmAABmMwBmZgBmmQBmzABm/zNmADNmMzNmZjNmmTNmzDNm/2ZmAGZm
M2ZmZmZmmWZmzGZm/5lmAJlmM5lmZplmmZlmzJlm/8xmAMxmM8xmZsxmmcxmzMxm//9mAP9m
M/9mZv9mmf9mzP9m/wCZAACZMwCZZgCZmQCZzACZ/zOZADOZMzOZZjOZmTOZzDOZ/2aZAGaZ
M2aZZmaZmWaZzGaZ/5mZAJmZM5mZZpmZmZmZzJmZ/8yZAMyZM8yZZsyZmcyZzMyZ//+ZAP+Z
M/+ZZv+Zmf+ZzP+Z/wDMAADMMwDMZgDMmQDMzADM/zPMADPMMzPMZjPMmTPMzDPM/2bMAGbM
M2bMZmbMmWbMzGbM/5nMAJnMM5nMZpnMmZnMzJnM/8zMAMzMM8zMZszMmczMzMzM///MAP/M
M//MZv/Mmf/MzP/M/wD/AAD/MwD/ZgD/mQD/zAD//zP/ADP/MzP/ZjP/mTP/zDP//2b/AGb/
M2b/Zmb/mWb/zGb//5n/AJn/M5n/Zpn/mZn/zJn//8z/AMz/M8z/Zsz/mcz/zMz/////AP//
M///Zv//mf//zP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACwALAAAILQCvCRQIYKBBgtUKHrwGoJpD
hQMbOnxoEIDFixAXLsR4seJEih5BHmyYceTBgAA7'
]! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 4/24/2005 05:36'!
buttonEventIcon
	"Icon for classic button events"
	^self tempEventIcon! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 2/12/2010 19:39'!
changeEventIcon
	^ChangeEventIcon ifNil:[
		ChangeEventIcon := CIconLibrary formFromEncodedImage: 
 'R0lGODlhCwALAPcAAAAAAAAAMwAAZgAAmQAAzAAA/zMAADMAMzMAZjMAmTMAzDMA/2YAAGYA
M2YAZmYAmWYAzGYA/5kAAJkAM5kAZpkAmZkAzJkA/8wAAMwAM8wAZswAmcwAzMwA//8AAP8A
M/8AZv8Amf8AzP8A/wAzAAAzMwAzZgAzmQAzzAAz/zMzADMzMzMzZjMzmTMzzDMz/2YzAGYz
M2YzZmYzmWYzzGYz/5kzAJkzM5kzZpkzmZkzzJkz/8wzAMwzM8wzZswzmcwzzMwz//8zAP8z
M/8zZv8zmf8zzP8z/wBmAABmMwBmZgBmmQBmzABm/zNmADNmMzNmZjNmmTNmzDNm/2ZmAGZm
M2ZmZmZmmWZmzGZm/5lmAJlmM5lmZplmmZlmzJlm/8xmAMxmM8xmZsxmmcxmzMxm//9mAP9m
M/9mZv9mmf9mzP9m/wCZAACZMwCZZgCZmQCZzACZ/zOZADOZMzOZZjOZmTOZzDOZ/2aZAGaZ
M2aZZmaZmWaZzGaZ/5mZAJmZM5mZZpmZmZmZzJmZ/8yZAMyZM8yZZsyZmcyZzMyZ//+ZAP+Z
M/+ZZv+Zmf+ZzP+Z/wDMAADMMwDMZgDMmQDMzADM/zPMADPMMzPMZjPMmTPMzDPM/2bMAGbM
M2bMZmbMmWbMzGbM/5nMAJnMM5nMZpnMmZnMzJnM/8zMAMzMM8zMZszMmczMzMzM///MAP/M
M//MZv/Mmf/MzP/M/wD/AAD/MwD/ZgD/mQD/zAD//zP/ADP/MzP/ZjP/mTP/zDP//2b/AGb/
M2b/Zmb/mWb/zGb//5n/AJn/M5n/Zpn/mZn/zJn//8z/AMz/M8z/Zsz/mcz/zMz/////AP//
M///Zv//mf//zP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACwALAAAIMwCvCRwIYKBBgQAKHmR1LaHD
gQwRKixo5aDCa1YqHhyYkeDBihMvRmzosOBIghc3phQYEAA7'
]! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 2/12/2010 19:39'!
defaultEventIcon
	^DefaultEventIcon ifNil:[
		DefaultEventIcon := CIconLibrary formFromEncodedImage: 
 'R0lGODlhCwALAPcAAAAAAAAAMwAAZgAAmQAAzAAA/zMAADMAMzMAZjMAmTMAzDMA/2YAAGYA
M2YAZmYAmWYAzGYA/5kAAJkAM5kAZpkAmZkAzJkA/8wAAMwAM8wAZswAmcwAzMwA//8AAP8A
M/8AZv8Amf8AzP8A/wAzAAAzMwAzZgAzmQAzzAAz/zMzADMzMzMzZjMzmTMzzDMz/2YzAGYz
M2YzZmYzmWYzzGYz/5kzAJkzM5kzZpkzmZkzzJkz/8wzAMwzM8wzZswzmcwzzMwz//8zAP8z
M/8zZv8zmf8zzP8z/wBmAABmMwBmZgBmmQBmzABm/zNmADNmMzNmZjNmmTNmzDNm/2ZmAGZm
M2ZmZmZmmWZmzGZm/5lmAJlmM5lmZplmmZlmzJlm/8xmAMxmM8xmZsxmmcxmzMxm//9mAP9m
M/9mZv9mmf9mzP9m/wCZAACZMwCZZgCZmQCZzACZ/zOZADOZMzOZZjOZmTOZzDOZ/2aZAGaZ
M2aZZmaZmWaZzGaZ/5mZAJmZM5mZZpmZmZmZzJmZ/8yZAMyZM8yZZsyZmcyZzMyZ//+ZAP+Z
M/+ZZv+Zmf+ZzP+Z/wDMAADMMwDMZgDMmQDMzADM/zPMADPMMzPMZjPMmTPMzDPM/2bMAGbM
M2bMZmbMmWbMzGbM/5nMAJnMM5nMZpnMmZnMzJnM/8zMAMzMM8zMZszMmczMzMzM///MAP/M
M//MZv/Mmf/MzP/M/wD/AAD/MwD/ZgD/mQD/zAD//zP/ADP/MzP/ZjP/mTP/zDP//2b/AGb/
M2b/Zmb/mWb/zGb//5n/AJn/M5n/Zpn/mZn/zJn//8z/AMz/M8z/Zsz/mcz/zMz/////AP//
M///Zv//mf//zP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACwALAAAIMACvCbwGoODAgwCoKQRwkKDC
hwwHJnxILaLAiRAbYrSI0GDDi4o4Igz5EaRIhA0DAgA7'
]! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 2/12/2010 19:40'!
downArrowIcon
	^DownArrow ifNil:[
		DownArrow := CIconLibrary formFromEncodedImage: 
 'R0lGODlhCwALAPcAAAAAAAAAMwAAZgAAmQAAzAAA/zMAADMAMzMAZjMAmTMAzDMA/2YAAGYA
M2YAZmYAmWYAzGYA/5kAAJkAM5kAZpkAmZkAzJkA/8wAAMwAM8wAZswAmcwAzMwA//8AAP8A
M/8AZv8Amf8AzP8A/wAzAAAzMwAzZgAzmQAzzAAz/zMzADMzMzMzZjMzmTMzzDMz/2YzAGYz
M2YzZmYzmWYzzGYz/5kzAJkzM5kzZpkzmZkzzJkz/8wzAMwzM8wzZswzmcwzzMwz//8zAP8z
M/8zZv8zmf8zzP8z/wBmAABmMwBmZgBmmQBmzABm/zNmADNmMzNmZjNmmTNmzDNm/2ZmAGZm
M2ZmZmZmmWZmzGZm/5lmAJlmM5lmZplmmZlmzJlm/8xmAMxmM8xmZsxmmcxmzMxm//9mAP9m
M/9mZv9mmf9mzP9m/wCZAACZMwCZZgCZmQCZzACZ/zOZADOZMzOZZjOZmTOZzDOZ/2aZAGaZ
M2aZZmaZmWaZzGaZ/5mZAJmZM5mZZpmZmZmZzJmZ/8yZAMyZM8yZZsyZmcyZzMyZ//+ZAP+Z
M/+ZZv+Zmf+ZzP+Z/wDMAADMMwDMZgDMmQDMzADM/zPMADPMMzPMZjPMmTPMzDPM/2bMAGbM
M2bMZmbMmWbMzGbM/5nMAJnMM5nMZpnMmZnMzJnM/8zMAMzMM8zMZszMmczMzMzM///MAP/M
M//MZv/Mmf/MzP/M/wD/AAD/MwD/ZgD/mQD/zAD//zP/ADP/MzP/ZjP/mTP/zDP//2b/AGb/
M2b/Zmb/mWb/zGb//5n/AJn/M5n/Zpn/mZn/zJn//8z/AMz/M8z/Zsz/mcz/zMz/////AP//
M///Zv//mf//zP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACwALAAAIKACvCRxIsKDBgQASKgRAEEC1
hw8ZNoQocWLEgw4rFsx4UKDGjiANBgQAOw=='
]! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 2/12/2010 19:40'!
flagIcon
	^FlagIcon ifNil:[
		FlagIcon := CIconLibrary formFromEncodedImage: 
 'R0lGODlhCwALAPcAAAAAAAAAMwAAZgAAmQAAzAAA/zMAADMAMzMAZjMAmTMAzDMA/2YAAGYA
M2YAZmYAmWYAzGYA/5kAAJkAM5kAZpkAmZkAzJkA/8wAAMwAM8wAZswAmcwAzMwA//8AAP8A
M/8AZv8Amf8AzP8A/wAzAAAzMwAzZgAzmQAzzAAz/zMzADMzMzMzZjMzmTMzzDMz/2YzAGYz
M2YzZmYzmWYzzGYz/5kzAJkzM5kzZpkzmZkzzJkz/8wzAMwzM8wzZswzmcwzzMwz//8zAP8z
M/8zZv8zmf8zzP8z/wBmAABmMwBmZgBmmQBmzABm/zNmADNmMzNmZjNmmTNmzDNm/2ZmAGZm
M2ZmZmZmmWZmzGZm/5lmAJlmM5lmZplmmZlmzJlm/8xmAMxmM8xmZsxmmcxmzMxm//9mAP9m
M/9mZv9mmf9mzP9m/wCZAACZMwCZZgCZmQCZzACZ/zOZADOZMzOZZjOZmTOZzDOZ/2aZAGaZ
M2aZZmaZmWaZzGaZ/5mZAJmZM5mZZpmZmZmZzJmZ/8yZAMyZM8yZZsyZmcyZzMyZ//+ZAP+Z
M/+ZZv+Zmf+ZzP+Z/wDMAADMMwDMZgDMmQDMzADM/zPMADPMMzPMZjPMmTPMzDPM/2bMAGbM
M2bMZmbMmWbMzGbM/5nMAJnMM5nMZpnMmZnMzJnM/8zMAMzMM8zMZszMmczMzMzM///MAP/M
M//MZv/Mmf/MzP/M/wD/AAD/MwD/ZgD/mQD/zAD//zP/ADP/MzP/ZjP/mTP/zDP//2b/AGb/
M2b/Zmb/mWb/zGb//5n/AJn/M5n/Zpn/mZn/zJn//8z/AMz/M8z/Zsz/mcz/zMz/////AP//
M///Zv//mf//zP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACwALAAAINACvCRwoEACAgQcNGhxx8BoA
hiMiRjToEKLEiRUfXmRY0CJDihkVEsw4kuDDhiVPlkRYMiAAOw=='
]! !

!TweakBrowser class methodsFor: 'class initialization' stamp: 'bf 5/10/2005 17:11'!
initialize
	SystemBrowser default: self.
	self initializeEventMap.! !

!TweakBrowser class methodsFor: 'class initialization' stamp: 'bf 5/10/2005 17:09'!
initializeEventMap
	"Initialize the event map"
	EventMap := Dictionary new.
	"Mouse events"
	#(
		mouseDown mouseEnter mouseMove mouseStillDown mouseLeave mouseUp
		mouseClick mouseDoubleClick mouseDrag
		dragEnter dragMove dragLeave dragDrop
		dropAccept dropReject
		redButtonDown redButtonMove redButtonUp
		yellowButtonDown yellowButtonMove yellowButtonUp
		blueButtonDown blueButtonMove blueButtonUp
	) do:[:event| EventMap at: event put: self mouseEventIcon].

	"Keyboard events"
	#(
		keyboardFocusEnter keyboardFocusLeave
		keyDown keyUp keyStroke
	) do:[:event| EventMap at: event put: self keyboardEventIcon].

	"Structure events"
	#(
		structureChanged frameChanged 
		layoutChanged containerLayoutChanged
		open close
		opened closed "remove those"
	) do:[:event| EventMap at: event put: self structureEventIcon].

	"Window events"
	#(
		closeRequest expandRequest collapseRequest activate passivate
	) do:[:event| EventMap at: event put: self windowEventIcon].

	"Text events"
	#(
		textModified textSelectionChanged save accept revert
	) do:[:event| EventMap at: event put: self textEventIcon].

	"Tick events"
	#(
		tick
	) do:[:event| EventMap at: event put: self tickEventIcon].

	"Button events"
	#(
		fire
	) do:[:event| EventMap at: event put: self buttonEventIcon].

	"Collection and list events"
	#(
		changed cursorChanged wrapCursorChanged
	) do:[:event| EventMap at: event put: self listEventIcon].
! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 2/12/2010 19:40'!
keyboardEventIcon
	^KeyboardEventIcon ifNil:[
		KeyboardEventIcon := CIconLibrary formFromEncodedImage: 
 'R0lGODlhCwALAPcAAAAAAAAAMwAAZgAAmQAAzAAA/zMAADMAMzMAZjMAmTMAzDMA/2YAAGYA
M2YAZmYAmWYAzGYA/5kAAJkAM5kAZpkAmZkAzJkA/8wAAMwAM8wAZswAmcwAzMwA//8AAP8A
M/8AZv8Amf8AzP8A/wAzAAAzMwAzZgAzmQAzzAAz/zMzADMzMzMzZjMzmTMzzDMz/2YzAGYz
M2YzZmYzmWYzzGYz/5kzAJkzM5kzZpkzmZkzzJkz/8wzAMwzM8wzZswzmcwzzMwz//8zAP8z
M/8zZv8zmf8zzP8z/wBmAABmMwBmZgBmmQBmzABm/zNmADNmMzNmZjNmmTNmzDNm/2ZmAGZm
M2ZmZmZmmWZmzGZm/5lmAJlmM5lmZplmmZlmzJlm/8xmAMxmM8xmZsxmmcxmzMxm//9mAP9m
M/9mZv9mmf9mzP9m/wCZAACZMwCZZgCZmQCZzACZ/zOZADOZMzOZZjOZmTOZzDOZ/2aZAGaZ
M2aZZmaZmWaZzGaZ/5mZAJmZM5mZZpmZmZmZzJmZ/8yZAMyZM8yZZsyZmcyZzMyZ//+ZAP+Z
M/+ZZv+Zmf+ZzP+Z/wDMAADMMwDMZgDMmQDMzADM/zPMADPMMzPMZjPMmTPMzDPM/2bMAGbM
M2bMZmbMmWbMzGbM/5nMAJnMM5nMZpnMmZnMzJnM/8zMAMzMM8zMZszMmczMzMzM///MAP/M
M//MZv/Mmf/MzP/M/wD/AAD/MwD/ZgD/mQD/zAD//zP/ADP/MzP/ZjP/mTP/zDP//2b/AGb/
M2b/Zmb/mWb/zGb//5n/AJn/M5n/Zpn/mZn/zJn//8z/AMz/M8z/Zsz/mcz/zMz/////AP//
M///Zv//mf//zP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACwALAAAIQACvWRlIkKDAa31OIFSYEICV
PgkhnpDo8ITFixcdLtzY8GHEhAsrYmR4QuO1kyA7Qlw5seNIjACuAZhJk+a1gAAAOw=='
]! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 4/24/2005 05:35'!
listEventIcon
	"Icon for classic list events"
	^self temp2EventIcon! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 2/12/2010 19:40'!
mouseEventIcon
	^MouseEventIcon ifNil:[
		MouseEventIcon := CIconLibrary formFromEncodedImage: 
 'R0lGODlhCwALAPcAAAAAAAAAMwAAZgAAmQAAzAAA/zMAADMAMzMAZjMAmTMAzDMA/2YAAGYA
M2YAZmYAmWYAzGYA/5kAAJkAM5kAZpkAmZkAzJkA/8wAAMwAM8wAZswAmcwAzMwA//8AAP8A
M/8AZv8Amf8AzP8A/wAzAAAzMwAzZgAzmQAzzAAz/zMzADMzMzMzZjMzmTMzzDMz/2YzAGYz
M2YzZmYzmWYzzGYz/5kzAJkzM5kzZpkzmZkzzJkz/8wzAMwzM8wzZswzmcwzzMwz//8zAP8z
M/8zZv8zmf8zzP8z/wBmAABmMwBmZgBmmQBmzABm/zNmADNmMzNmZjNmmTNmzDNm/2ZmAGZm
M2ZmZmZmmWZmzGZm/5lmAJlmM5lmZplmmZlmzJlm/8xmAMxmM8xmZsxmmcxmzMxm//9mAP9m
M/9mZv9mmf9mzP9m/wCZAACZMwCZZgCZmQCZzACZ/zOZADOZMzOZZjOZmTOZzDOZ/2aZAGaZ
M2aZZmaZmWaZzGaZ/5mZAJmZM5mZZpmZmZmZzJmZ/8yZAMyZM8yZZsyZmcyZzMyZ//+ZAP+Z
M/+ZZv+Zmf+ZzP+Z/wDMAADMMwDMZgDMmQDMzADM/zPMADPMMzPMZjPMmTPMzDPM/2bMAGbM
M2bMZmbMmWbMzGbM/5nMAJnMM5nMZpnMmZnMzJnM/8zMAMzMM8zMZszMmczMzMzM///MAP/M
M//MZv/Mmf/MzP/M/wD/AAD/MwD/ZgD/mQD/zAD//zP/ADP/MzP/ZjP/mTP/zDP//2b/AGb/
M2b/Zmb/mWb/zGb//5n/AJn/M5n/Zpn/mZn/zJn//8z/AMz/M8z/Zsz/mcz/zMz/////AP//
M///Zv//mf//zP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACwALAAAIOQCvCRQIYKDBgQUPEgTAMCHC
ZxAhOgTwzImTYBaDJaSYMePGihc9EgSJ8eLHYChTnkyJcmLDhgIDAgA7'
]! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 2/12/2010 19:40'!
noEventIcon
	^NoEventIcon ifNil:[
		NoEventIcon := CIconLibrary formFromEncodedImage: 
 'R0lGODlhCwALAPcAAAAAAAAAMwAAZgAAmQAAzAAA/zMAADMAMzMAZjMAmTMAzDMA/2YAAGYA
M2YAZmYAmWYAzGYA/5kAAJkAM5kAZpkAmZkAzJkA/8wAAMwAM8wAZswAmcwAzMwA//8AAP8A
M/8AZv8Amf8AzP8A/wAzAAAzMwAzZgAzmQAzzAAz/zMzADMzMzMzZjMzmTMzzDMz/2YzAGYz
M2YzZmYzmWYzzGYz/5kzAJkzM5kzZpkzmZkzzJkz/8wzAMwzM8wzZswzmcwzzMwz//8zAP8z
M/8zZv8zmf8zzP8z/wBmAABmMwBmZgBmmQBmzABm/zNmADNmMzNmZjNmmTNmzDNm/2ZmAGZm
M2ZmZmZmmWZmzGZm/5lmAJlmM5lmZplmmZlmzJlm/8xmAMxmM8xmZsxmmcxmzMxm//9mAP9m
M/9mZv9mmf9mzP9m/wCZAACZMwCZZgCZmQCZzACZ/zOZADOZMzOZZjOZmTOZzDOZ/2aZAGaZ
M2aZZmaZmWaZzGaZ/5mZAJmZM5mZZpmZmZmZzJmZ/8yZAMyZM8yZZsyZmcyZzMyZ//+ZAP+Z
M/+ZZv+Zmf+ZzP+Z/wDMAADMMwDMZgDMmQDMzADM/zPMADPMMzPMZjPMmTPMzDPM/2bMAGbM
M2bMZmbMmWbMzGbM/5nMAJnMM5nMZpnMmZnMzJnM/8zMAMzMM8zMZszMmczMzMzM///MAP/M
M//MZv/Mmf/MzP/M/wD/AAD/MwD/ZgD/mQD/zAD//zP/ADP/MzP/ZjP/mTP/zDP//2b/AGb/
M2b/Zmb/mWb/zGb//5n/AJn/M5n/Zpn/mZn/zJn//8z/AMz/M8z/Zsz/mcz/zMz/////AP//
M///Zv//mf//zP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACwALAAAIFQCvCRxIsKDBgwgTKlzIsKHD
h9cCAgA7'
]! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 2/12/2010 19:41'!
structureEventIcon
	^StructureEventIcon ifNil:[
		StructureEventIcon := CIconLibrary formFromEncodedImage: 
 'R0lGODlhCwALAPcAAAAAAAAAMwAAZgAAmQAAzAAA/zMAADMAMzMAZjMAmTMAzDMA/2YAAGYA
M2YAZmYAmWYAzGYA/5kAAJkAM5kAZpkAmZkAzJkA/8wAAMwAM8wAZswAmcwAzMwA//8AAP8A
M/8AZv8Amf8AzP8A/wAzAAAzMwAzZgAzmQAzzAAz/zMzADMzMzMzZjMzmTMzzDMz/2YzAGYz
M2YzZmYzmWYzzGYz/5kzAJkzM5kzZpkzmZkzzJkz/8wzAMwzM8wzZswzmcwzzMwz//8zAP8z
M/8zZv8zmf8zzP8z/wBmAABmMwBmZgBmmQBmzABm/zNmADNmMzNmZjNmmTNmzDNm/2ZmAGZm
M2ZmZmZmmWZmzGZm/5lmAJlmM5lmZplmmZlmzJlm/8xmAMxmM8xmZsxmmcxmzMxm//9mAP9m
M/9mZv9mmf9mzP9m/wCZAACZMwCZZgCZmQCZzACZ/zOZADOZMzOZZjOZmTOZzDOZ/2aZAGaZ
M2aZZmaZmWaZzGaZ/5mZAJmZM5mZZpmZmZmZzJmZ/8yZAMyZM8yZZsyZmcyZzMyZ//+ZAP+Z
M/+ZZv+Zmf+ZzP+Z/wDMAADMMwDMZgDMmQDMzADM/zPMADPMMzPMZjPMmTPMzDPM/2bMAGbM
M2bMZmbMmWbMzGbM/5nMAJnMM5nMZpnMmZnMzJnM/8zMAMzMM8zMZszMmczMzMzM///MAP/M
M//MZv/Mmf/MzP/M/wD/AAD/MwD/ZgD/mQD/zAD//zP/ADP/MzP/ZjP/mTP/zDP//2b/AGb/
M2b/Zmb/mWb/zGb//5n/AJn/M5n/Zpn/mZn/zJn//8z/AMz/M8z/Zsz/mcz/zMz/////AP//
M///Zv//mf//zP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACwALAAAIQQABCBwI4JrBawDgKCxQoODB
hNHgMHRoMOFCghUZNpTAUUJBABMlyPLQkSAACR5IcqR4suPKhy5fwix58OFAgwEBADs='
]! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 2/12/2010 19:41'!
temp2EventIcon
	^Temp2EventIcon ifNil:[
		Temp2EventIcon := CIconLibrary formFromEncodedImage: 
 'R0lGODlhCwALAPcAAAAAAAAAMwAAZgAAmQAAzAAA/zMAADMAMzMAZjMAmTMAzDMA/2YAAGYA
M2YAZmYAmWYAzGYA/5kAAJkAM5kAZpkAmZkAzJkA/8wAAMwAM8wAZswAmcwAzMwA//8AAP8A
M/8AZv8Amf8AzP8A/wAzAAAzMwAzZgAzmQAzzAAz/zMzADMzMzMzZjMzmTMzzDMz/2YzAGYz
M2YzZmYzmWYzzGYz/5kzAJkzM5kzZpkzmZkzzJkz/8wzAMwzM8wzZswzmcwzzMwz//8zAP8z
M/8zZv8zmf8zzP8z/wBmAABmMwBmZgBmmQBmzABm/zNmADNmMzNmZjNmmTNmzDNm/2ZmAGZm
M2ZmZmZmmWZmzGZm/5lmAJlmM5lmZplmmZlmzJlm/8xmAMxmM8xmZsxmmcxmzMxm//9mAP9m
M/9mZv9mmf9mzP9m/wCZAACZMwCZZgCZmQCZzACZ/zOZADOZMzOZZjOZmTOZzDOZ/2aZAGaZ
M2aZZmaZmWaZzGaZ/5mZAJmZM5mZZpmZmZmZzJmZ/8yZAMyZM8yZZsyZmcyZzMyZ//+ZAP+Z
M/+ZZv+Zmf+ZzP+Z/wDMAADMMwDMZgDMmQDMzADM/zPMADPMMzPMZjPMmTPMzDPM/2bMAGbM
M2bMZmbMmWbMzGbM/5nMAJnMM5nMZpnMmZnMzJnM/8zMAMzMM8zMZszMmczMzMzM///MAP/M
M//MZv/Mmf/MzP/M/wD/AAD/MwD/ZgD/mQD/zAD//zP/ADP/MzP/ZjP/mTP/zDP//2b/AGb/
M2b/Zmb/mWb/zGb//5n/AJn/M5n/Zpn/mZn/zJn//8z/AMz/M8z/Zsz/mcz/zMz/////AP//
M///Zv//mf//zP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACwALAAAISgCvCRwIoODAgwBaKQRAsGCr
PhBPMARwrCIAiH0kUtSlq+DFjAWPdaxGEoDEaxQBVJPGkhBDgQVXtnwJU6Y0lwdREtpJE6HB
gQEBADs='
]! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 2/12/2010 19:41'!
tempEventIcon
	^TempEventIcon ifNil:[
		TempEventIcon := CIconLibrary formFromEncodedImage: 
'R0lGODlhCwALAPcAAAAAAAAAMwAAZgAAmQAAzAAA/zMAADMAMzMAZjMAmTMAzDMA/2YAAGYA
M2YAZmYAmWYAzGYA/5kAAJkAM5kAZpkAmZkAzJkA/8wAAMwAM8wAZswAmcwAzMwA//8AAP8A
M/8AZv8Amf8AzP8A/wAzAAAzMwAzZgAzmQAzzAAz/zMzADMzMzMzZjMzmTMzzDMz/2YzAGYz
M2YzZmYzmWYzzGYz/5kzAJkzM5kzZpkzmZkzzJkz/8wzAMwzM8wzZswzmcwzzMwz//8zAP8z
M/8zZv8zmf8zzP8z/wBmAABmMwBmZgBmmQBmzABm/zNmADNmMzNmZjNmmTNmzDNm/2ZmAGZm
M2ZmZmZmmWZmzGZm/5lmAJlmM5lmZplmmZlmzJlm/8xmAMxmM8xmZsxmmcxmzMxm//9mAP9m
M/9mZv9mmf9mzP9m/wCZAACZMwCZZgCZmQCZzACZ/zOZADOZMzOZZjOZmTOZzDOZ/2aZAGaZ
M2aZZmaZmWaZzGaZ/5mZAJmZM5mZZpmZmZmZzJmZ/8yZAMyZM8yZZsyZmcyZzMyZ//+ZAP+Z
M/+ZZv+Zmf+ZzP+Z/wDMAADMMwDMZgDMmQDMzADM/zPMADPMMzPMZjPMmTPMzDPM/2bMAGbM
M2bMZmbMmWbMzGbM/5nMAJnMM5nMZpnMmZnMzJnM/8zMAMzMM8zMZszMmczMzMzM///MAP/M
M//MZv/Mmf/MzP/M/wD/AAD/MwD/ZgD/mQD/zAD//zP/ADP/MzP/ZjP/mTP/zDP//2b/AGb/
M2b/Zmb/mWb/zGb//5n/AJn/M5n/Zpn/mZn/zJn//8z/AMz/M8z/Zsz/mcz/zMz/////AP//
M///Zv//mf//zP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACwALAAAIOQCvCRxIkCCAgwcLAqjGkCGA
gQulSZRI6OG1iBOlVRSIceLGi9UyarTYkaLFi4RSpjzJESHLggUDAgA7'
]! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 2/12/2010 19:41'!
textEventIcon
	^TextEventIcon ifNil:[
		TextEventIcon := CIconLibrary formFromEncodedImage: 
'R0lGODlhDAAMAPcAAAAAAAAAMwAAZgAAmQAAzAAA/zMAADMAMzMAZjMAmTMAzDMA/2YAAGYA
M2YAZmYAmWYAzGYA/5kAAJkAM5kAZpkAmZkAzJkA/8wAAMwAM8wAZswAmcwAzMwA//8AAP8A
M/8AZv8Amf8AzP8A/wAzAAAzMwAzZgAzmQAzzAAz/zMzADMzMzMzZjMzmTMzzDMz/2YzAGYz
M2YzZmYzmWYzzGYz/5kzAJkzM5kzZpkzmZkzzJkz/8wzAMwzM8wzZswzmcwzzMwz//8zAP8z
M/8zZv8zmf8zzP8z/wBmAABmMwBmZgBmmQBmzABm/zNmADNmMzNmZjNmmTNmzDNm/2ZmAGZm
M2ZmZmZmmWZmzGZm/5lmAJlmM5lmZplmmZlmzJlm/8xmAMxmM8xmZsxmmcxmzMxm//9mAP9m
M/9mZv9mmf9mzP9m/wCZAACZMwCZZgCZmQCZzACZ/zOZADOZMzOZZjOZmTOZzDOZ/2aZAGaZ
M2aZZmaZmWaZzGaZ/5mZAJmZM5mZZpmZmZmZzJmZ/8yZAMyZM8yZZsyZmcyZzMyZ//+ZAP+Z
M/+ZZv+Zmf+ZzP+Z/wDMAADMMwDMZgDMmQDMzADM/zPMADPMMzPMZjPMmTPMzDPM/2bMAGbM
M2bMZmbMmWbMzGbM/5nMAJnMM5nMZpnMmZnMzJnM/8zMAMzMM8zMZszMmczMzMzM///MAP/M
M//MZv/Mmf/MzP/M/wD/AAD/MwD/ZgD/mQD/zAD//zP/ADP/MzP/ZjP/mTP/zDP//2b/AGb/
M2b/Zmb/mWb/zGb//5n/AJn/M5n/Zpn/mZn/zJn//8z/AMz/M8z/Zsz/mcz/zMz/////AP//
M///Zv//mf//zP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAADAAMAAAIPAADCRwo8JrBawUTBjqI0GDB
hgcXIgRAsSLFiBczAsBo8OK1jQ5BegSJUKJHhwQDnWR4cCXLjyRZWrwYEAA7'
]! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 2/12/2010 19:41'!
tickEventIcon
	^TickEventIcon ifNil:[
		TickEventIcon := CIconLibrary formFromEncodedImage: 
'R0lGODlhDAAMAPcAAAAAAAAAMwAAZgAAmQAAzAAA/zMAADMAMzMAZjMAmTMAzDMA/2YAAGYA
M2YAZmYAmWYAzGYA/5kAAJkAM5kAZpkAmZkAzJkA/8wAAMwAM8wAZswAmcwAzMwA//8AAP8A
M/8AZv8Amf8AzP8A/wAzAAAzMwAzZgAzmQAzzAAz/zMzADMzMzMzZjMzmTMzzDMz/2YzAGYz
M2YzZmYzmWYzzGYz/5kzAJkzM5kzZpkzmZkzzJkz/8wzAMwzM8wzZswzmcwzzMwz//8zAP8z
M/8zZv8zmf8zzP8z/wBmAABmMwBmZgBmmQBmzABm/zNmADNmMzNmZjNmmTNmzDNm/2ZmAGZm
M2ZmZmZmmWZmzGZm/5lmAJlmM5lmZplmmZlmzJlm/8xmAMxmM8xmZsxmmcxmzMxm//9mAP9m
M/9mZv9mmf9mzP9m/wCZAACZMwCZZgCZmQCZzACZ/zOZADOZMzOZZjOZmTOZzDOZ/2aZAGaZ
M2aZZmaZmWaZzGaZ/5mZAJmZM5mZZpmZmZmZzJmZ/8yZAMyZM8yZZsyZmcyZzMyZ//+ZAP+Z
M/+ZZv+Zmf+ZzP+Z/wDMAADMMwDMZgDMmQDMzADM/zPMADPMMzPMZjPMmTPMzDPM/2bMAGbM
M2bMZmbMmWbMzGbM/5nMAJnMM5nMZpnMmZnMzJnM/8zMAMzMM8zMZszMmczMzMzM///MAP/M
M//MZv/Mmf/MzP/M/wD/AAD/MwD/ZgD/mQD/zAD//zP/ADP/MzP/ZjP/mTP/zDP//2b/AGb/
M2b/Zmb/mWb/zGb//5n/AJn/M5n/Zpn/mZn/zJn//8z/AMz/M8z/Zsz/mcz/zMz/////AP//
M///Zv//mf//zP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAADAAMAAAIOQCvCQRAkKDAgwcLIkwIYJFD
gwkdLmr4MOJDiQAGSpy4MSPFixivfQQ5UWPHkiY3ohz4EeJChQgDAgA7'
]! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 2/12/2010 19:41'!
upArrowIcon
	^UpArrow ifNil:[
		UpArrow := CIconLibrary formFromEncodedImage: 'R0lGODlhCwALAPcAAAAAAAAAMwAAZgAAmQAAzAAA/zMAADMAMzMAZjMAmTMAzDMA/2YAAGYA
M2YAZmYAmWYAzGYA/5kAAJkAM5kAZpkAmZkAzJkA/8wAAMwAM8wAZswAmcwAzMwA//8AAP8A
M/8AZv8Amf8AzP8A/wAzAAAzMwAzZgAzmQAzzAAz/zMzADMzMzMzZjMzmTMzzDMz/2YzAGYz
M2YzZmYzmWYzzGYz/5kzAJkzM5kzZpkzmZkzzJkz/8wzAMwzM8wzZswzmcwzzMwz//8zAP8z
M/8zZv8zmf8zzP8z/wBmAABmMwBmZgBmmQBmzABm/zNmADNmMzNmZjNmmTNmzDNm/2ZmAGZm
M2ZmZmZmmWZmzGZm/5lmAJlmM5lmZplmmZlmzJlm/8xmAMxmM8xmZsxmmcxmzMxm//9mAP9m
M/9mZv9mmf9mzP9m/wCZAACZMwCZZgCZmQCZzACZ/zOZADOZMzOZZjOZmTOZzDOZ/2aZAGaZ
M2aZZmaZmWaZzGaZ/5mZAJmZM5mZZpmZmZmZzJmZ/8yZAMyZM8yZZsyZmcyZzMyZ//+ZAP+Z
M/+ZZv+Zmf+ZzP+Z/wDMAADMMwDMZgDMmQDMzADM/zPMADPMMzPMZjPMmTPMzDPM/2bMAGbM
M2bMZmbMmWbMzGbM/5nMAJnMM5nMZpnMmZnMzJnM/8zMAMzMM8zMZszMmczMzMzM///MAP/M
M//MZv/Mmf/MzP/M/wD/AAD/MwD/ZgD/mQD/zAD//zP/ADP/MzP/ZjP/mTP/zDP//2b/AGb/
M2b/Zmb/mWb/zGb//5n/AJn/M5n/Zpn/mZn/zJn//8z/AMz/M8z/Zsz/mcz/zMz/////AP//
M///Zv//mf//zP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACwALAAAIKgCvCRxIsKBBgwAOCgRQLeFB
hg4LMqzWECHFigQnXox4DYDHjxwVihwYEAA7'
]! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 2/12/2010 19:42'!
viewEventIcon
	^ViewEventIcon ifNil:[
		ViewEventIcon := CIconLibrary formFromEncodedImage: 
 'R0lGODlhCwALAPcAAAAAAAAAMwAAZgAAmQAAzAAA/zMAADMAMzMAZjMAmTMAzDMA/2YAAGYA
M2YAZmYAmWYAzGYA/5kAAJkAM5kAZpkAmZkAzJkA/8wAAMwAM8wAZswAmcwAzMwA//8AAP8A
M/8AZv8Amf8AzP8A/wAzAAAzMwAzZgAzmQAzzAAz/zMzADMzMzMzZjMzmTMzzDMz/2YzAGYz
M2YzZmYzmWYzzGYz/5kzAJkzM5kzZpkzmZkzzJkz/8wzAMwzM8wzZswzmcwzzMwz//8zAP8z
M/8zZv8zmf8zzP8z/wBmAABmMwBmZgBmmQBmzABm/zNmADNmMzNmZjNmmTNmzDNm/2ZmAGZm
M2ZmZmZmmWZmzGZm/5lmAJlmM5lmZplmmZlmzJlm/8xmAMxmM8xmZsxmmcxmzMxm//9mAP9m
M/9mZv9mmf9mzP9m/wCZAACZMwCZZgCZmQCZzACZ/zOZADOZMzOZZjOZmTOZzDOZ/2aZAGaZ
M2aZZmaZmWaZzGaZ/5mZAJmZM5mZZpmZmZmZzJmZ/8yZAMyZM8yZZsyZmcyZzMyZ//+ZAP+Z
M/+ZZv+Zmf+ZzP+Z/wDMAADMMwDMZgDMmQDMzADM/zPMADPMMzPMZjPMmTPMzDPM/2bMAGbM
M2bMZmbMmWbMzGbM/5nMAJnMM5nMZpnMmZnMzJnM/8zMAMzMM8zMZszMmczMzMzM///MAP/M
M//MZv/Mmf/MzP/M/wD/AAD/MwD/ZgD/mQD/zAD//zP/ADP/MzP/ZjP/mTP/zDP//2b/AGb/
M2b/Zmb/mWb/zGb//5n/AJn/M5n/Zpn/mZn/zJn//8z/AMz/M8z/Zsz/mcz/zMz/////AP//
M///Zv//mf//zP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACwALAAAIPwCvCQRAkKDAgwBYKVQIYCDB
hQwLWpkokeI1AFYARINzAmPDixnhcPQ4MOOJkyRBZiRoMSFEiAVjyjxIk2ZAAAA7'
]! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 2/12/2010 19:42'!
warningIcon
	^WarningIcon ifNil:[
		WarningIcon := CIconLibrary formFromEncodedImage: 
'R0lGODlhCwALAPcAAAAAAAAAMwAAZgAAmQAAzAAA/zMAADMAMzMAZjMAmTMAzDMA/2YAAGYA
M2YAZmYAmWYAzGYA/5kAAJkAM5kAZpkAmZkAzJkA/8wAAMwAM8wAZswAmcwAzMwA//8AAP8A
M/8AZv8Amf8AzP8A/wAzAAAzMwAzZgAzmQAzzAAz/zMzADMzMzMzZjMzmTMzzDMz/2YzAGYz
M2YzZmYzmWYzzGYz/5kzAJkzM5kzZpkzmZkzzJkz/8wzAMwzM8wzZswzmcwzzMwz//8zAP8z
M/8zZv8zmf8zzP8z/wBmAABmMwBmZgBmmQBmzABm/zNmADNmMzNmZjNmmTNmzDNm/2ZmAGZm
M2ZmZmZmmWZmzGZm/5lmAJlmM5lmZplmmZlmzJlm/8xmAMxmM8xmZsxmmcxmzMxm//9mAP9m
M/9mZv9mmf9mzP9m/wCZAACZMwCZZgCZmQCZzACZ/zOZADOZMzOZZjOZmTOZzDOZ/2aZAGaZ
M2aZZmaZmWaZzGaZ/5mZAJmZM5mZZpmZmZmZzJmZ/8yZAMyZM8yZZsyZmcyZzMyZ//+ZAP+Z
M/+ZZv+Zmf+ZzP+Z/wDMAADMMwDMZgDMmQDMzADM/zPMADPMMzPMZjPMmTPMzDPM/2bMAGbM
M2bMZmbMmWbMzGbM/5nMAJnMM5nMZpnMmZnMzJnM/8zMAMzMM8zMZszMmczMzMzM///MAP/M
M//MZv/Mmf/MzP/M/wD/AAD/MwD/ZgD/mQD/zAD//zP/ADP/MzP/ZjP/mTP/zDP//2b/AGb/
M2b/Zmb/mWb/zGb//5n/AJn/M5n/Zpn/mZn/zJn//8z/AMz/M8z/Zsz/mcz/zMz/////AP//
M///Zv//mf//zP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACwALAAAINACvCbwGoODAgwCkKQRwUGDC
hQ0fKpTGcODDixYnSqy4USPBiSAXSqQIsmDIkgQLqlR5LSAAOw=='
]! !

!TweakBrowser class methodsFor: 'accessing' stamp: 'ar 2/12/2010 19:42'!
windowEventIcon
	^WindowEventIcon ifNil:[
		WindowEventIcon := CIconLibrary formFromEncodedImage: 
'R0lGODlhCwALAPcAAAAAAAAAMwAAZgAAmQAAzAAA/zMAADMAMzMAZjMAmTMAzDMA/2YAAGYA
M2YAZmYAmWYAzGYA/5kAAJkAM5kAZpkAmZkAzJkA/8wAAMwAM8wAZswAmcwAzMwA//8AAP8A
M/8AZv8Amf8AzP8A/wAzAAAzMwAzZgAzmQAzzAAz/zMzADMzMzMzZjMzmTMzzDMz/2YzAGYz
M2YzZmYzmWYzzGYz/5kzAJkzM5kzZpkzmZkzzJkz/8wzAMwzM8wzZswzmcwzzMwz//8zAP8z
M/8zZv8zmf8zzP8z/wBmAABmMwBmZgBmmQBmzABm/zNmADNmMzNmZjNmmTNmzDNm/2ZmAGZm
M2ZmZmZmmWZmzGZm/5lmAJlmM5lmZplmmZlmzJlm/8xmAMxmM8xmZsxmmcxmzMxm//9mAP9m
M/9mZv9mmf9mzP9m/wCZAACZMwCZZgCZmQCZzACZ/zOZADOZMzOZZjOZmTOZzDOZ/2aZAGaZ
M2aZZmaZmWaZzGaZ/5mZAJmZM5mZZpmZmZmZzJmZ/8yZAMyZM8yZZsyZmcyZzMyZ//+ZAP+Z
M/+ZZv+Zmf+ZzP+Z/wDMAADMMwDMZgDMmQDMzADM/zPMADPMMzPMZjPMmTPMzDPM/2bMAGbM
M2bMZmbMmWbMzGbM/5nMAJnMM5nMZpnMmZnMzJnM/8zMAMzMM8zMZszMmczMzMzM///MAP/M
M//MZv/Mmf/MzP/M/wD/AAD/MwD/ZgD/mQD/zAD//zP/ADP/MzP/ZjP/mTP/zDP//2b/AGb/
M2b/Zmb/mWb/zGb//5n/AJn/M5n/Zpn/mZn/zJn//8z/AMz/M8z/Zsz/mcz/zMz/////AP//
M///Zv//mf//zP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACwALAAAIMgCvCRxIUCCAgwgRGqTEsCEl
AAYTJjQIraJFaBCvAbhoMeNGjhgpgvQIMmREiQVTCgwIADs='
]! !

!TweakBrowser methodsFor: 'private' stamp: 'ar 5/9/2005 17:17'!
changed: what
	what == #messageList ifTrue:[messageListCache := nil].
	^super changed: what! !

!TweakBrowser methodsFor: 'drawing' stamp: 'ar 5/12/2005 21:59'!
drawMessage: index in: itemBox on: aCanvas
	"Drawing support. Draw an item in the message list."
	| selector class icon itemColor |
	class := self selectedClassOrMetaClass.
	selector := self messageList at: index ifAbsent:[^self].
	icon := self iconForClass: class selector: selector.
	icon ifNotNil:[
		aCanvas paintImage: icon at: itemBox leftCenter - (-2@icon height // 2)
	].

	itemColor := index = messageListIndex
			ifTrue:[Color red] 
			ifFalse:[Color black].
	aCanvas drawString: selector asString
			in: (itemBox insetBy: (15@0 corner: 0@0)) 
			font: Preferences standardListFont 
			color: itemColor.! !

!TweakBrowser methodsFor: 'drawing' stamp: 'ar 6/19/2005 13:41'!
iconForClass: class selector: selector
	| trigger up down method icon |
	method := class compiledMethodAt: selector ifAbsent:[^nil].
	(method propertyValueAt: #warning) ifNotNil:[^TweakBrowser warningIcon].
	(method propertyValueAt: #error) ifNotNil:[^TweakBrowser warningIcon].
	trigger := self methodTriggerFor: selector in: class.
	trigger ifNil:[
		(method hasLiteral: #flag:) ifTrue:[^TweakBrowser flagIcon].
		up := self isThisAnOverride: selector in: class.
		down := self isThereAnOverride: selector in: class.
		(up and:[down]) 
			ifTrue:[^TweakBrowser bothArrowIcon]
			ifFalse:[
				up ifTrue:[^TweakBrowser upArrowIcon].
				down ifTrue:[^TweakBrowser downArrowIcon].
			].
	] ifNotNil:[
		icon := EventMap at: trigger ifAbsent:[nil].
		icon ifNotNil:[^icon].
		(trigger endsWith: 'changed' caseSensitive: false) 
			ifTrue:[^TweakBrowser changeEventIcon].
		(method hasLiteral: #flag:) ifTrue:[^TweakBrowser flagIcon].
		^TweakBrowser defaultEventIcon
	].
	^nil! !

!TweakBrowser methodsFor: 'accessing' stamp: 'ar 5/16/2005 14:08'!
isThereAnOverride: selector in: aClass
	"Answer whether any subclass of my selected class implements my 
	selected selector"
	| set |
true ifTrue:[
	aClass allSubclassesDo: [ :cls | (cls includesSelector: selector) ifTrue: [ ^true ]].
	^false.
] ifFalse:["later, use cache like here but we need to be able to flush it properly"
	overriddenClassCache ifNil:[overriddenClassCache := Dictionary new].
	set := overriddenClassCache at: aClass ifAbsentPut:[
		set := Set new.
		aClass allSubclassesDo: [ :cls | set addAll: cls selectors].
		set].
	^set includes: selector
].! !

!TweakBrowser methodsFor: 'accessing' stamp: 'ar 4/24/2005 08:33'!
isThisAnOverride: selector in: aClass
	"Answer whether any superclass of my selected class implements my selected selector"
	aClass allSuperclassesDo: [ :cls | (cls includesSelector: selector) ifTrue: [ ^true ]].
	^ false! !

!TweakBrowser methodsFor: 'accessing' stamp: 'ar 5/10/2005 13:33'!
messageList
	"Would be nice if we could cache this like here but then list validation no longer works and I cannot figure out how to make it work. So for now this is just very, very slow."
	^messageListCache ifNil:[self messageListRaw]! !

!TweakBrowser methodsFor: 'accessing' stamp: 'ar 5/10/2005 13:33'!
messageListRaw
	^messageListCache := super messageList! !

!TweakBrowser methodsFor: 'accessing' stamp: 'ar 4/24/2005 04:15'!
methodTriggerFor: sel in: myClass
	| lkupClass trigger |
	(myClass includesBehavior: CProtoObject) ifFalse:[^nil].
	lkupClass := myClass.
	[lkupClass == CProtoObject] whileFalse:[
		trigger := lkupClass methodTriggerAt: sel.
		trigger ifNotNil:[^trigger eventName].
		lkupClass := lkupClass superclass.
	].
	^nil! !

!FillInTheBlank class methodsFor: '*Tweak-Morphic-override' stamp: 'ar 1/5/2005 18:08'!
fillInTheBlankMorphClass
	"By factoring out this class references, it becomes possible to discard 
	morphic by simply removing this class.  All calls to this method needs
	to be protected by 'Smalltalk isMorphic' tests."

	^self ifTweak:[CDialog] ifNot:[FillInTheBlankMorph]! !

PluggableListMorph subclass: #PluggableTweakMessageListMorph
	instanceVariableNames: 'drawAction getRawListSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Morphic-DevTools'!

!PluggableTweakMessageListMorph methodsFor: 'as yet unclassified' stamp: 'ar 5/9/2005 17:01'!
drawAction
	^drawAction! !

!PluggableTweakMessageListMorph methodsFor: 'as yet unclassified' stamp: 'ar 5/9/2005 17:01'!
drawAction: aMessage
	drawAction := aMessage! !

!PluggableTweakMessageListMorph methodsFor: 'as yet unclassified' stamp: 'ar 5/10/2005 13:37'!
getRawList
	"Answer the list to be displayed.  Caches the returned list in the 'list' ivar"
	getRawListSelector ifNil:[^self getList].
	^(model perform: getRawListSelector) collect:[ :item | item asStringOrText].! !

!PluggableTweakMessageListMorph methodsFor: 'as yet unclassified' stamp: 'ar 5/10/2005 13:31'!
getRawListSelector
	^getRawListSelector! !

!PluggableTweakMessageListMorph methodsFor: 'as yet unclassified' stamp: 'ar 5/10/2005 13:32'!
getRawListSelector: aSelector
	getRawListSelector := aSelector! !

!PluggableTweakMessageListMorph methodsFor: 'as yet unclassified' stamp: 'ar 4/24/2005 04:45'!
listMorphClass
	^TweakMessageListMorph! !

!PluggableTweakMessageListMorph methodsFor: 'as yet unclassified' stamp: 'ar 5/10/2005 13:30'!
verifyContents
	"Verify the contents of the receiver, reconstituting if necessary.  Called whenever window is reactivated, to react to possible structural changes.  Also called periodically in morphic if the smartUpdating preference is true"
	| newList existingSelection anIndex oldList |
	oldList := list ifNil: [ #() ].
	newList := self getRawList.
	((oldList == newList) "fastest" or: [oldList = newList]) ifTrue: [^ self].
	self flash.  "list has changed beneath us; give the user a little visual feedback that the contents of the pane are being updated."
	existingSelection := self selectionIndex > 0 ifTrue: [ oldList at: self selectionIndex ] ifFalse: [ nil ].
	self updateList.
	(existingSelection notNil and: [(anIndex := list indexOf: existingSelection asStringOrText ifAbsent: [nil]) notNil])
		ifTrue:
			[model noteSelectionIndex: anIndex for: getListSelector.
			self selectionIndex: anIndex]
		ifFalse:
			[self changeModelSelection: 0]! !

BorderedMorph subclass: #CRootMorph
	instanceVariableNames: 'window myWorld lastMouseEvent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Morphic'!

!CRootMorph class methodsFor: 'as yet unclassified' stamp: 'ar 6/18/2005 16:42'!
initialize
	"CRootMorph initialize."
	TheWorldMenu unregisterOpenCommandWithReceiver: self.! !

!CRootMorph methodsFor: 'events' stamp: 'ar 4/28/2005 19:45'!
aboutToEnterWorld
	"Morphic is about to enter the World that I belong to."


	self resumeTweakWorld! !

!CRootMorph methodsFor: 'events' stamp: 'jcg 4/28/2005 22:19'!
aboutToLeaveWorld
	"Morphic is about to leave the World that I belong to."

	self suspendTweakWorld! !

!CRootMorph methodsFor: 'drawing' stamp: 'ar 12/19/2002 05:46'!
areasRemainingToFill: aRectangle
	(color isColor and: [color isTranslucent]) ifTrue: [^ Array with: aRectangle].
	self wantsRoundedCorners
		ifTrue: [^ aRectangle areasOutside: self boundsWithinCorners]
		ifFalse: [^ aRectangle areasOutside: self bounds]! !

!CRootMorph methodsFor: 'event handling' stamp: 'ar 8/28/2005 20:55'!
convertDropEvent: anEvent
	^CDropFilesEvent new 
			setPosition: anEvent position 
			contents: anEvent contents
			hand: nil
! !

!CRootMorph methodsFor: 'event handling' stamp: 'ar 8/28/2005 20:55'!
convertKeyboardEvent: anEvent
	^CKeyboardEvent new
		setType: anEvent type
		buttons: anEvent buttons
		position: anEvent position
		keyValue: anEvent keyValue
		hand: nil
		stamp: anEvent timeStamp! !

!CRootMorph methodsFor: 'event handling' stamp: 'ar 8/28/2005 20:54'!
convertMorphicEvent: anEvent
	"Convert a MorphicEvent"
	anEvent isMorphicEvent ifFalse:[^anEvent].
	anEvent isMouse ifTrue:[^self convertMouseEvent: anEvent].
	anEvent isKeyboard ifTrue:[^self convertKeyboardEvent: anEvent].
	anEvent type == #dropFilesEvent ifTrue:[^self convertDropEvent: anEvent].
	^nil! !

!CRootMorph methodsFor: 'event handling' stamp: 'jdm 4/1/2008 11:21'!
convertMouseEvent: anEvent
	anEvent class == MouseMoveEvent ifTrue:[
		^CMouseMoveEvent new 
			setType: anEvent type 
			startPoint: anEvent startPoint
			endPoint: anEvent endPoint
			trail: anEvent trail
			buttons: anEvent buttons
			hand: nil
			stamp: anEvent timeStamp.
	].
	anEvent class == MouseButtonEvent ifTrue:[
		^CMouseButtonEvent new 
			setType: anEvent type 
			position: anEvent position
			which: anEvent whichButton
			buttons: anEvent buttons
			nrClicks: anEvent nrClicks
			hand: nil
			stamp: anEvent timeStamp.
	].
	^nil! !

!CRootMorph methodsFor: 'initialize' stamp: 'bf 5/7/2004 13:47'!
defaultBackgroundColor
	^CWidgetLibrary default baseColor! !

!CRootMorph methodsFor: 'drawing' stamp: 'ar 4/20/2005 14:25'!
drawOn: aCanvas
	aCanvas fillRectangle: self bounds fillStyle: Color transparent borderStyle: self borderStyle.
	aCanvas clipBy: self innerBounds during:[:cc|
		cc drawImage: myWorld display at: (self innerBounds origin).
	].
	self isStepping ifFalse:[
		aCanvas frameAndFillRectangle: (bounds center - (100@25) extent: 200@50)
			fillColor: Color red borderWidth: 2 borderColor: Color black.
		aCanvas drawString: 'SCHEDULER STOPPED!!' at: bounds center - (60@15).
		aCanvas drawString: 'Click to restart' at: bounds center - (50@-5).
	].
! !

!CRootMorph methodsFor: 'geometry' stamp: 'ar 11/10/2004 17:38'!
extent: newExtent
	super extent: newExtent.
	myWorld ifNotNil:[
		myWorld bootstrap:[myWorld displayExtent: self innerBounds extent].
	].! !

!CRootMorph methodsFor: 'accessing' stamp: 'ar 12/18/2004 19:49'!
fullScreenMode
	^owner ~~ window! !

!CRootMorph methodsFor: 'accessing' stamp: 'ar 6/24/2005 21:09'!
fullScreenMode: aBool
	aBool = self fullScreenMode ifTrue:[^self].
	aBool ifTrue:[
		self world addMorphFront: self.
		self borderWidth: 0.
		self topLeft: 0@0.
		self extent: DisplayScreen actualScreenSize.
	] ifFalse:[
		window addMorphBack: self.
		self borderWidth: 2.
	].! !

!CRootMorph methodsFor: 'event handling' stamp: 'jdm 4/2/2008 14:50'!
handleEvent: anEvent
	| evt |
	(anEvent type == #mouseDown and:[anEvent redButtonChanged]) ifTrue:[
		self isStepping ifFalse:[
			self changed.
			^self startStepping]].
	lastMouseEvent ifNil:[lastMouseEvent := ActiveHand lastEvent].
	(#(mouseDown mouseUp mouseMove mouseEnter mouseLeave
	keyDown keyUp keystroke
	dropFilesEvent) includes: anEvent type) ifTrue:[
		(anEvent isMouse) ifTrue:[
			anEvent hand keyboardFocus: self.
			anEvent isMouseDown ifTrue:[
				anEvent hand mouseFocus: self.
				self comeToFront].
			anEvent isMouseUp ifTrue:[
				anEvent hand releaseMouseFocus: self.
			].
		].
		"pass it into myWorld"
		evt := anEvent translatedBy: self innerBounds origin negated.
		(evt isMouse and:[evt isMouseMove and:[lastMouseEvent isMouse and: [evt buttons ~= lastMouseEvent buttons]]]) ifTrue:[
			"fix up event type"
			(lastMouseEvent mouseButtons bitClear: evt mouseButtons) > 0 ifTrue:[
				evt := CMouseButtonEvent new 
					setType: #mouseUp 
					position: evt position
					which: (lastMouseEvent buttons bitClear: evt buttons)
					buttons: evt buttons
					hand: evt hand
					stamp: evt timeStamp.
			].
			(evt mouseButtons bitClear: lastMouseEvent mouseButtons) > 0 ifTrue:[
				evt := CMouseButtonEvent new 
					setType: #mouseDown
					position: evt position
					which: (evt buttons bitClear: lastMouseEvent buttons)
					buttons: evt buttons
					nrClicks: evt nrClicks
					hand: evt hand
					stamp: evt timeStamp.
			].
		].
		lastMouseEvent := anEvent clone.
		evt type == #mouseMove ifTrue:[
				evt setTrail: (evt trail collect:[:pt| pt - bounds origin]).
		].
		"evt printString displayAt: 0@0."
		evt := self convertMorphicEvent: evt.
		evt ifNotNil:[myWorld eventQueue nextPut: evt].
		anEvent wasHandled: true.
		^true].
	^super handleEvent: anEvent! !

!CRootMorph methodsFor: 'event handling' stamp: 'ar 12/8/2002 01:51'!
handlerForMouseDown: anEvent
	^self! !

!CRootMorph methodsFor: 'event handling' stamp: 'ar 12/8/2002 02:57'!
handlesMouseOver: evt
	^true! !

!CRootMorph methodsFor: 'initialize' stamp: 'ar 3/28/2004 01:56'!
initialExtent
	^800@600! !

!CRootMorph methodsFor: 'initialize' stamp: 'ar 5/28/2005 14:46'!
initialize
	super initialize.
	color := Color transparent.
	self extent: 800@600.
	self center: World center.
	"self cornerStyle: #rounded.
	self addDropShadow."
	myWorld := CWorldPlayer new.
	Processor activeIsland project: myWorld.
	myWorld display: (Form extent: self extent depth: Display depth).
	myWorld ownerMorph: self.
	myWorld eventQueue: SharedQueue new.
! !

!CRootMorph methodsFor: 'initialize' stamp: 'jcg 4/28/2005 22:18'!
intoWorld: aWorld
	"The receiver has just appeared in a new world."

	aWorld ifNil: [^ self].
	super intoWorld: aWorld.
	aWorld 
		when: #aboutToLeaveWorld send: #aboutToLeaveWorld to: self;
		when: #aboutToEnterWorld send: #aboutToEnterWorld to: self.
! !

!CRootMorph methodsFor: 'accessing' stamp: 'ar 12/7/2003 18:15'!
label
	window ifNil:[^''].
	^window label! !

!CRootMorph methodsFor: 'accessing' stamp: 'ar 12/7/2003 18:18'!
label: aString
	window ifNil:[^nil].
	window setLabel: aString! !

!CRootMorph methodsFor: 'initialize' stamp: 'ar 4/20/2005 14:28'!
modelSleep
	"This is useful when we pop up a debugger"
	self isStepping ifFalse:[self changed].! !

!CRootMorph methodsFor: 'initialize' stamp: 'ar 4/20/2005 14:29'!
modelWakeUp
	"This is useful when we pop up a debugger"
	self isStepping ifFalse:[self changed].! !

!CRootMorph methodsFor: 'accessing' stamp: 'ar 12/18/2004 19:53'!
morphicLayerNumber
	self fullScreenMode ifTrue:[^20] ifFalse:[^super morphicLayerNumber].! !

!CRootMorph methodsFor: 'event handling' stamp: 'ar 12/8/2002 02:57'!
mouseEnter: evt
	evt hand newKeyboardFocus: self.! !

!CRootMorph methodsFor: 'accessing' stamp: 'ar 1/9/2003 16:12'!
myWorld
	^myWorld! !

!CRootMorph methodsFor: 'initialize' stamp: 'jcg 4/28/2005 22:18'!
outOfWorld: aWorld
	"The receiver has just left aWorld."

	aWorld ifNil: [^ self].
	aWorld removeActionsWithReceiver: self.
	self suspendTweakWorld.
	super outOfWorld: aWorld..! !

!CRootMorph methodsFor: 'private' stamp: 'ar 5/28/2005 14:38'!
resumeTweakWorld
	"Notify our world that it is being resumed"
	Processor activeIsland project: myWorld.
	myWorld activeProject signal: #resume.
	self step.! !

!CRootMorph methodsFor: 'stepping' stamp: 'ar 1/8/2008 17:49'!
step
	| rectList offset |
	Processor activeIsland project: myWorld.
	myWorld signalTick: #tick.
	rectList := myWorld runMorphicCycle.
	offset := self innerBounds origin.
	rectList do:[:dmgRect|
		self invalidRect: (dmgRect translateBy: offset).
	].! !

!CRootMorph methodsFor: 'stepping' stamp: 'ar 8/17/2003 16:04'!
stepAt: millisecondClockValue in: aWindow
	"do nothing"! !

!CRootMorph methodsFor: 'stepping' stamp: 'ar 9/8/2003 21:10'!
stepTime
	^20! !

!CRootMorph methodsFor: 'private' stamp: 'ar 5/28/2005 14:39'!
suspendTweakWorld
	"Notify our world that it is being suspended."
	Processor activeIsland project: myWorld.
	myWorld activeProject signal: #suspend.
	self step.! !

!CRootMorph methodsFor: 'stepping' stamp: 'ar 12/8/2002 00:59'!
wantsSteps
	^true! !

!CRootMorph methodsFor: 'accessing' stamp: 'ar 12/7/2003 18:13'!
window
	^window! !

!CRootMorph methodsFor: 'accessing' stamp: 'ar 12/7/2003 18:13'!
window: aWindow
	window := aWindow! !

!ParagraphEditor methodsFor: '*Tweak-Morphic' stamp: 'eem 1/6/2009 13:54'!
saveHtmlInFileUseBreaks: useBreaks "<Boolean>"
	| fileName stringToSave parentWindow labelToUse |
	stringToSave := paragraph text asHtmlTextStyle: paragraph textStyle useBreaks: useBreaks.
	stringToSave size == 0 ifTrue: [^ self inform: 'nothing to save.'].
	parentWindow := self model dependents
						detect: [:dep | dep isKindOf: SystemWindow]
						ifNone: [nil].
	labelToUse := parentWindow isNil
					ifTrue: ['Untitled']
					ifFalse: [parentWindow label].
	(labelToUse beginsWith: 'Binary:') ifTrue:
		[labelToUse := labelToUse allButFirst: 7].
	labelToUse := labelToUse withBlanksTrimmed.
	fileName := FillInTheBlank
					request: 'File name? (".html" will be added to end)' 
					initialAnswer: labelToUse.
	fileName size == 0 ifTrue: [^ self beep].
	(fileName asLowercase endsWith: '.text') ifFalse: [fileName := fileName,'.html'].

	(FileStream newFileNamed: fileName)
		nextPutAll: stringToSave; close! !

!CMenu class methodsFor: '*tweak-morphic' stamp: 'bf 11/29/2004 15:41'!
displayMVCMenu: mvcMenuMorph at: aPoint during: aBlock

	| menu |
	menu := self fromMVCMenu: mvcMenuMorph.
	menu open.
	aBlock ensure: [menu close]
	! !

!CMenu class methodsFor: '*tweak-morphic' stamp: 'bf 11/29/2004 15:39'!
fromMVCMenu: mvcMenuMorph

	| menu |
	menu := self new.
	(mvcMenuMorph submorphWithProperty: #titleString) ifNotNilDo: [:title |
		menu label: title submorphs first contents].
	mvcMenuMorph items do: [:item |
		menu add: item contents
			target: item target
			action: item selector
			arguments: item arguments].
	^menu! !

!CMenu class methodsFor: '*tweak-morphic' stamp: 'bf 11/29/2004 15:41'!
invokeMVCMenu: mvcMenuMorph at: aPoint

	| menu |
	menu := self fromMVCMenu: mvcMenuMorph.
	menu run.
! !

!TextMorphEditor methodsFor: '*Tweak-Morphic-override' stamp: 'ar 5/5/2004 19:10'!
yellowButtonDown: event
	"Process a yellow button event. Answer true if the event was handled, false otherwise."
	(paragraph attributesAt: event cursorPoint) do:[:attr|
		attr menu ifNotNil:[
			attr menu openAt: event cursorPoint.
			^true]].
	^false! !

!ChangeSet methodsFor: '*Tweak-Morphic-override' stamp: 'eem 1/28/2009 16:28'!
fatDefForClass: class

	| newDef oldDef oldStrm newStrm outStrm oldVars newVars addedVars |
	(class includesBehavior: CProtoObject) ifTrue:[^class definition].
	newDef := class definition.
	oldDef := (self changeRecorderFor: class) priorDefinition.
	oldDef ifNil: [^ newDef].
	oldDef = newDef ifTrue: [^ newDef].

	oldStrm := ReadStream on: oldDef.
	newStrm := ReadStream on: newDef.
	outStrm := WriteStream on: (String new: newDef size * 2).

	"Merge inst vars from old and new defs..."
	oldStrm upToAll: 'instanceVariableNames'; upTo: $'.
	outStrm 
		nextPutAll: (newStrm upToAll: 'instanceVariableNames'); 
		nextPutAll: 'instanceVariableNames:'.
	newStrm peek = $: ifTrue: [newStrm next].	"may or may not be there, but already written"
	outStrm nextPutAll: (newStrm upTo: $'); nextPut: $'.
	oldVars := (oldStrm upTo: $') findTokens: Character separators.
	newVars := (newStrm upTo: $') findTokens: Character separators.
	addedVars := oldVars asSet addAll: newVars; removeAll: oldVars; asOrderedCollection.
	oldVars , addedVars
		do: [:var | outStrm nextPutAll: var]
		separatedBy: [outStrm space].
	outStrm nextPut: $'.

	class isMeta ifFalse:
		["Merge class vars from old and new defs..."
		oldStrm upToAll: 'classVariableNames:'; upTo: $'.
		outStrm nextPutAll: (newStrm upToAll: 'classVariableNames:'); nextPutAll: 'classVariableNames:';
			nextPutAll: (newStrm upTo: $'); nextPut: $'.
		oldVars := (oldStrm upTo: $') findTokens: Character separators.
		newVars := (newStrm upTo: $') findTokens: Character separators.
		addedVars := oldVars asSet addAll: newVars; removeAll: oldVars; asOrderedCollection.
		oldVars , addedVars
			do: [:var | outStrm nextPutAll: var]
			separatedBy: [outStrm space].
		outStrm nextPut: $'].

	outStrm nextPutAll: newStrm upToEnd.
	^ outStrm contents
! !

!PluggableTextMorph methodsFor: '*Tweak-Morphic' stamp: 'eem 6/14/2008 13:49'!
saveHtmlInFileUseBreaks: useBreaks "<Boolean>"
	self handleEdit: [textMorph editor saveHtmlInFileUseBreaks: useBreaks]! !

!NewParagraph methodsFor: '*Tweak-Morphic' stamp: 'ar 5/5/2004 19:10'!
attributesAt: aPoint
	"Answer the attributes at the given point"
	^text attributesAt: (self characterBlockAtPoint: aPoint) stringIndex forStyle: textStyle! !
CProjectInMorphic initialize!
TweakBrowser initialize!
CRootMorph initialize!
